/* iRiver iFP 7XX Emulator
   Copyright (C) 2005 Tomasz Malesinski <tmal@mimuw.edu.pl>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef _HARDWARE_H_
#define _HARDWARE_H_

#include "emuriver.h"
#include <stdio.h>
#include "event.h"

struct gpio_handler {
  void *data;
  int (*get)(void *data, int port, int bit);
  void (*set)(void *data, int port, int bit, int value);
};

struct dma_channel {
  uint32 src;
  uint32 dest;
  uint32 len;
  uint32 r0c;
  uint32 r10;
  uint32 r1c;
};

struct dma_ctrl {
  uint32 intstat;
  uint32 inten;
  struct dma_channel channel[8];
  struct hardware *hardware;
};

struct codec {
  uint32 volume;
};

struct clocks {
  uint32 reg0[12];
  uint32 reg30[12];
  uint32 reg60[12];
  uint32 reg90[12];
  uint32 regc0[73];
  uint32 reg308[67];
  uint32 reg414[3];
  uint32 reg420[18];
};

struct hardware {
  uint32 gpio_dir[8];
  uint32 gpio_out[8];
  struct gpio_handler gpio_handler[8][32];

  struct flash *flash[4];
  struct prog_flash *prog_flash;

  struct lcd *lcd;
  struct usb *usb;
  int usb_connected;

  uint32 irq_reg[0x140];
  uint32 irq_request;

  uint32 timer_period;
  int timer_active;
  struct event ev_timer_irq;

  struct dma_ctrl dma_ctrl;
  struct event ev_dma_irq;
  struct codec codec;
  struct clocks clocks;

  struct memory *phys_memory;
  struct memory *memory;
  int mmu[16];

  char log_buf[128];
  int log_ptr;
};

struct prog_flash {
  int state;
  FILE *fp;
  uint8 *content;
};

extern unsigned int tick;

int tick_before(unsigned int a, unsigned int b);

int gpio_get_bit(struct hardware *hardware, int port, int bit);
void gpio_install_handler(struct hardware *hardware, int port, int bit,
			  void *data,
			  int (*get)(void *, int, int),
			  void (*set)(void *, int, int, int));

void install_hardware(struct hardware *hardware, struct memory *memory);
void hardware_reset(struct hardware *hardware);
struct hardware *create_hardware(void);
void hardware_connect_flash(struct hardware *hardware,
			    int n, struct flash *flash);

int check_irq(void *data);

#endif
