/* iRiver iFP 7XX Emulator
   Copyright (C) 2005 Tomasz Malesinski <tmal@mimuw.edu.pl>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include <stdio.h>
#include <stdarg.h>

#include "log.h"

static FILE *log_fp = NULL;
static int log_indent = 0;

static int log_level[] = {
  50, /* LOG_GENERAL */
   5, /* LOG_GPIO */
  10, /* LOG_FLASH */
   5, /* LOG_LCD */
   5, /* LOG_FW_FLASH */
  10, /* LOG_MEMORY */
   5, /* LOG_ADC */
   5, /* LOG_TRACE */
  10, /* LOG_GDB */
  10, /* LOG_I2C */
  10, /* LOG_CPU */
  10, /* LOG_USB */
};

int log_check(int source, int level) {
  return log_fp && level <= log_level[source];
}

void log_set_fp(FILE *fp) {
  log_fp = fp;
  log_indent = 0;
}

int logmsg(int source, int level, const char *format, ...) {
  va_list ap;
  int r, i;

  if (!log_check(source, level))
    return 0;

  for (i = 0; i < log_indent; i++)
    fputc(' ', log_fp);

  va_start(ap, format);
  r = vfprintf(log_fp, format, ap);
  va_end(ap);
  return r;
}

void log_reset_indent(void) {
  log_indent = 0;
}

void log_inc_indent(void) {
  log_indent++;
}

void log_dec_indent(void) {
  log_indent--;
}
