/* iRiver iFP 7XX Emulator
   Copyright (C) 2005 Tomasz Malesinski <tmal@mimuw.edu.pl>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef _MEMORY_H_
#define _MEMORY_H_

#include "emuriver.h"

struct mem_region {
  uint8 (*read)(void *data, uint32 adr);
  void (*write)(void *data, uint32 adr, uint8 val);
  uint32 (*read_half_word)(void *data, uint32 adr);
  void (*write_half_word)(void *data, uint32 adr, uint32 val);
  uint32 (*read_word)(void *data, uint32 adr);
  void (*write_word)(void *data, uint32 adr, uint32 val);
  void *data;
};

struct mem_block {
  uint8 *mem;
  uint32 *profile;
  uint32 mask;
  uint32 len;
  int rom;
  int profiling;
};

struct memory {
  struct mem_region region[1 << 16];
};

struct memory *create_memory(void);
struct mem_block *create_memory_area_rom(struct memory *mem, uint32 adr,
					 uint32 len, char *file);
struct mem_block *create_memory_area_ram(struct memory *mem, uint32 adr,
					 uint32 len);
void free_memory_block(struct mem_block *block);
void dump_profile(FILE *fp, struct mem_block *block, uint32 offset);

uint32 memory_read_word(struct memory *, uint32 adr);
void memory_write_word(struct memory *, uint32 adr, uint32 val);

uint8 read_void(void *data, uint32 adr);
uint32 read_half_word_void(void *data, uint32 adr);
uint32 read_word_void(void *data, uint32 adr);
void write_void(void *data, uint32 adr, uint8 val);
void write_half_word_void(void *data, uint32 adr, uint32 val);
void write_word_void(void *data, uint32 adr, uint32 val);

void memory_copy_configuration(struct memory *dest_memory,
			       uint32 dest_start, uint32 dest_end,
			       struct memory *src_memory,
			       uint32 src_start);
#endif
