/* iRiver iFP 7XX Emulator
   Copyright (C) 2005 Tomasz Malesinski <tmal@mimuw.edu.pl>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "emuriver.h"

static uint8 memory_snapshot[0x10000];
static uint8 possible[0x10000];
static int started;
static uint8 *memory_block;

void trainer_init(uint8 *memory) {
  started = 0;
  memory_block = memory;
}

void trainer_start(void) {
  int i;
  
  for (i = 0; i < 0x10000; i++) {
    memory_snapshot[i] = memory_block[i];
    possible[i] = 1;
  }
  started = 1;
}

void trainer_value_changed(void) {
  int i;

  if (!started)
    return;

  for (i = 0; i < 0x10000; i++) {
    if (memory_snapshot[i] == memory_block[i])
      possible[i] = 0;
    memory_snapshot[i] = memory_block[i];
  }
}

void trainer_value_not_changed(void) {
  int i;

  if (!started)
    return;

  for (i = 0; i < 0x10000; i++) {
    if (memory_snapshot[i] != memory_block[i])
      possible[i] = 0;
    memory_snapshot[i] = memory_block[i];
  }
}

int trainer_n_possible(void) {
  int i, n;
  if (!started)
    return -1;

  for (i = 0; i < 0x10000; i++)
    if (possible[i]) n++;
  return n;
}

int trainer_get_possible(int *p, int p_len) {
  int i, n;

  if (!started)
    return -1;

  n = 0;
  for (i = 0; i < 0x10000 && n < p_len; i++)
    if (possible[i]) p[n++] = i;
  return n;
}
