/* iRiver iFP 7XX Emulator
   Copyright (C) 2005 Tomasz Malesinski <tmal@mimuw.edu.pl>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#ifndef _USB_H_
#define _USB_H_

#include "emuriver.h"

#define USB_ERR_ENDPOINT_DISABLED -1
#define USB_ERR_BUFFER_FULL -2

#define USB_BUF_SIZE 1024

struct usb_in_fifo {
  int ptr;
  int len;
  uint8 data[USB_BUF_SIZE];
};

struct usb_endpoint {
  uint32 buflen;
  uint8 ctrl;
  uint32 maxpacketsize;
  uint32 type;
  struct usb_in_fifo in_fifo;

  uint8 out_buf[USB_BUF_SIZE];
  int out_ptr;
};

struct usb {
  struct usb_endpoint endpoint[16];
  uint32 mode_reg;
  uint32 inten_reg;
  uint32 int_status;
  uint8 intconf_reg;
  uint8 index_reg;

  struct usb_in_fifo setup_in_fifo;

  uint8 in_buf[USB_BUF_SIZE];
  int in_buf_ptr;
};

struct usb *create_usb(void);
void free_usb(struct usb *usb);

void usb_reset(struct usb *usb);

uint8 usb_read(void *data, uint32 adr);
void usb_write(void *data, uint32 adr, uint8 val);
uint32 usb_read_half_word(void *data, uint32 adr);
void usb_write_half_word(void *data, uint32 adr, uint32 val);
uint32 usb_read_word(void *data, uint32 adr);
void usb_write_word(void *data, uint32 adr, uint32 val);

void usb_receive_byte(struct usb *usb, uint8 byte);
int usb_rcv_setup(struct usb *usb);
int usb_rcv_data(struct usb *usb, int e);
int usb_bus_reset(struct usb *usb);
int usb_suspend(struct usb *usb);
int usb_resume(struct usb *usb);
int usb_hs_stat(struct usb *usb);

#endif
